/*
 * IgnitionController.java
 *
 * Created on May 14, 2003, 6:24 PM
 */

package org.megaJoltLiteJr;

/**
 *
 * @author  brent
 */
import java.io.*;
import java.util.*;
import javax.comm.*;

public class IgnitionController{
    
    private String serialPortName=null;
    private int baudRate=4800;
    private boolean connected=false;
    
    /** Holds value of property serialPort. */
    private SerialPort serialPort;
    
    /** Creates a new instance of IgnitionController */
    public IgnitionController() {
    }

    public void connect(){
        
        openPort();
        setConnected(true);
    }
    
    public void disconnect(){
    
        SerialPort port = getSerialPort();

        if (null!=port){
            System.err.println("Closing serial port");
            port.removeEventListener();
            port.close();
        }
        setSerialPort(null);
        setConnected(false);
    }
    
    private void openPort(){
        
        CommPortIdentifier portId;
        Enumeration portList = CommPortIdentifier.getPortIdentifiers();
     
        String port = this.getSerialPortName();
        if (null==port) throw new IllegalStateException("Serial port is not set");
        
        SerialPort serialPort=null;
        
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier)portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                if (portId.getName().equals(port)) {
                    try {
                        serialPort = (SerialPort)portId.open("megaJoltLiteJr", 2000);
                    } catch (PortInUseException e) {throw new IllegalStateException("Port " + port + " is in use");}
                }
            }
        }
        
        if (null==serialPort) throw new CommRuntimeException("Unable to open port " + port + "; unknown serial port");
        
        try {
            serialPort.setSerialPortParams(getBaudRate(),
            SerialPort.DATABITS_8,
            SerialPort.STOPBITS_1,
            SerialPort.PARITY_NONE);
            //serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_XONXOFF_IN & SerialPort.FLOWCONTROL_XONXOFF_OUT);
            serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
            
        }
        catch (UnsupportedCommOperationException e) {
            throw new CommRuntimeException("Could not set serial port options: unsupported port Operation Exception " + e.getClass().getName() + " caught: " + e.getMessage());
        }
        
        this.setSerialPort(serialPort);
    }    
    
    
    
    public IgnitionState getIgnitionState(){
     
        if (!isConnected()) connect();
     
        IgnitionStateReader reader= new IgnitionStateReader( getSerialPort());
        
        IgnitionState currentState = reader.readIgnitionState();
        
        
        return currentState;
        
    }
    
    public IgnitionConfig getIgnitionConfig(){
        
        if (!isConnected()) connect();
        
        IgnitionConfigReader reader = new IgnitionConfigReader(getSerialPort());
        IgnitionConfig currentConfig = reader.readIgnitionConfig();
        reader.releaseSerialPort();
        return currentConfig;
    }
    
    public void updateIgnitionConfig(IgnitionConfig config){
        
        if (!isConnected()) connect();
        
        IgnitionConfigUpdater updater = new IgnitionConfigUpdater(getSerialPort());
        updater.updateIgnitionConfig(config);
        updater.releaseSerialPort();
    }
    
    public void writeFlash(){
        
        if (!isConnected()) connect();
        
        IgnitionConfigUpdater updater = new IgnitionConfigUpdater(getSerialPort());
        updater.writeFlash();
        updater.releaseSerialPort();
        
    }
    /** Getter for property serialPort.
     * @return Value of property serialPort.
     *
     */
    public String getSerialPortName() {
        return this.serialPortName;
    }    
    
    /** Setter for property serialPort.
     * @param serialPort New value of property serialPort.
     *
     */
    public void setSerialPortName(String serialPortName) {
        this.serialPortName = serialPortName;
    }    
    
    /** Getter for property baudRate.
     * @return Value of property baudRate.
     *
     */
    public int getBaudRate() {
        return this.baudRate;
    }
    
    /** Setter for property baudRate.
     * @param baudRate New value of property baudRate.
     *
     */
    public void setBaudRate(int baudRate) {
        this.baudRate = baudRate;
    }
    
    /** Getter for property connected.
     * @return Value of property connected.
     *
     */
    public boolean isConnected() {
        return this.connected;
    }
    
    /** Setter for property connected.
     * @param connected New value of property connected.
     *
     */
    public void setConnected(boolean connected) {
        this.connected = connected;
    }
    
    public void serialEvent(javax.comm.SerialPortEvent serialPortEvent) {
    }
    
    /** Getter for property serialPort.
     * @return Value of property serialPort.
     *
     */
    public SerialPort getSerialPort() {
        return this.serialPort;
    }
    
    /** Setter for property serialPort.
     * @param serialPort New value of property serialPort.
     *
     */
    public void setSerialPort(SerialPort serialPort) {
        this.serialPort = serialPort;
    }
 
    public void finalize(){
        disconnect();
    }
}
