/*
 * IgnitionStateReader.java
 *
 * Created on May 14, 2003, 7:05 PM
 */

package org.megaJoltLiteJr;

/**
 *
 * @author  brent
 */
import javax.comm.*;
import java.util.*;

import java.io.IOException;

public class IgnitionStateReader extends BaseControllerCommunicator {
    
    public static final String READ_STATE_COMMAND="S";
    public static final long TIMEOUT=2000;
    public static final int READ_DATA_LENGTH=6;
    
    public IgnitionStateReader(SerialPort serialPort){
        super(serialPort);
    }

    public IgnitionState readIgnitionState(){
    
        long timeoutAt= System.currentTimeMillis()+IgnitionStateReader.TIMEOUT;
        
        this.getInputBuffer().clear();
        
        try{
            //super.getSerialOutputStream().write(READ_STATE_COMMAND.getBytes());
            super.getSerialOutputStream().write(83);
        }
        catch (IOException e){
            throw new CommRuntimeException("Unable to send read Request",e);
        }
        
        while ( System.currentTimeMillis() < timeoutAt && !isReadComplete()){
            try{
                
                Thread.sleep(10);
            }
            catch (InterruptedException e){}
        }
        
      
        
        IgnitionState currentState = createCurrentState();
        
        return currentState;
    }
    

    
    private IgnitionState createCurrentState(){

        
        List readData = super.getInputBuffer();
        
        if (readData.size()<READ_DATA_LENGTH) throw new IllegalStateException("Read buffer length not correct: was " + readData.size() + "; expected " + READ_DATA_LENGTH);
        
        IgnitionState currentState = new IgnitionState();
        
        Byte ignitionAdvance = (Byte)readData.get(0);
        currentState.setIgnitionAdvance((int)ignitionAdvance.byteValue());
        
        int rpmHigh=getUnsignedByteValue(((Byte)readData.get(1)).byteValue());
        int rpmLow=getUnsignedByteValue(((Byte)readData.get(2)).byteValue());
        currentState.setRpmValue((rpmHigh*256)+rpmLow);
        
        int rpmBin = getUnsignedByteValue(((Byte)readData.get(3)).byteValue());
        currentState.setRpmBin(rpmBin);

        int mapBin = getUnsignedByteValue(((Byte)readData.get(4)).byteValue());
        currentState.setMapBin(mapBin);
        
        int mapValue = getUnsignedByteValue(((Byte)readData.get(5)).byteValue());
        currentState.setMapValue(mapValue);
        
        //System.out.println("value" + readData.get(5)+  " " +currentState.getMapValue());
        
        return currentState;
    }
    
    /**
     *
     *
     */
    public boolean isReadComplete(){

        
        List buffer = super.getInputBuffer();
        
        boolean readIsComplete = (buffer.size()>=READ_DATA_LENGTH);

        return readIsComplete;
    }
    
    public void dataReceived(List inBuffer) {
        
        
    }
    
}
