/*
 * MegaJoltLiteJr.java
 *
 * Created on May 17, 2003, 12:56 AM
 */

package org.megaJoltLiteJr.presentation;

import org.megaJoltLiteJr.*;

import java.util.*;
import java.util.prefs.Preferences;
import javax.swing.*;

import java.io.*;
import java.awt.Color;
import java.awt.Dimension;

/**
 *
 * @author  brent
 */
public class MegaJoltLiteJr extends javax.swing.JFrame {

    public static final int CONTROLLER_BAUD_RATE=4800;
    public static final int DEFAULT_CYLINDERS=4;

    public static final String LAST_DIRECTORY_PATH="lastDirPath";
    
    private static final int MIN_ADVANCE_VALUE=0;
    private static final int MAX_ADVANCE_VALUE=IgnitionConfig.MAX_IGNITION_ADVANCE_VALUE;
    private static final int ADVANCE_STEP_VALUE=1;
    private static final int MIN_MAP_VALUE=IgnitionConfig.MIN_MAP_BIN_VALUE;
    private static final int MAX_MAP_VALUE=IgnitionConfig.MAX_MAP_BIN_VALUE;
    private static final int MAP_STEP_VALUE=1;
    private static final int MIN_RPM_VALUE=IgnitionConfig.MIN_RPM_BIN_VALUE*100;
    private static final int MAX_RPM_VALUE=IgnitionConfig.MAX_RPM_BIN_VALUE*100;
    private static final int RPM_STEP_VALUE=100;
    private static final int IGNITION_GRID_ROWS=10;
    private static final int IGNITION_GRID_COLS=10;
    private static final Color RPM_BIN_BACKGROUND_COLOR=new Color(255,231,75);
    private static final Color MAP_BIN_BACKGROUND_COLOR=new Color(255,182,104);
    private static final Color ADVANCE_BACKGROUND_COLOR=new Color(253,255,222);
    private static final String TITLE_BAR_TEXT="MegaJolt Lite Jr.";
    
    private List mapBinSpinnerList = new ArrayList();
    private List rpmBinSpinnerList = new ArrayList();
    private List ignitionAdvanceSpinnerList = new ArrayList();
    private List userOutputConfigList = new ArrayList();
    private File loadedConfigFile=null;
    private IgnitionController ignitionController;
    
    /** Creates new form MegaJoltLiteJr */
    public MegaJoltLiteJr() {
        initLookAndFeel();
        initComponents();
        initIgnitionTableComponents();
        initUserOutputComponents();
        updateTitleBar();

        reloadConfiguration();
    }
    
    protected synchronized IgnitionController getIgnitionController(){
        
        if (ignitionController==null) {
            ignitionController=new IgnitionController();
            ignitionController.setSerialPortName(Options.getSerialPort());
            ignitionController.setBaudRate(CONTROLLER_BAUD_RATE);
            ignitionController.connect();
        }
        
        return ignitionController;
    }

    private static Preferences getPreferences(){
        return Preferences.userNodeForPackage(MegaJoltLiteJr.class);
    }
 
    public void finalize(){
        System.err.println("Disconnecting Ignition Controller");
        IgnitionController controller = getIgnitionController();
        if (null!=ignitionController) controller.disconnect();
    }
    private void initLookAndFeel(){
        
        try{
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Throwable t){
            System.err.println("Could not set Look and Feel: " + t.getMessage());
        }
    }
    private void setLoadedConfigFile(File newFile){
        loadedConfigFile=newFile;
        updateTitleBar();
    }
    
    private File getLoadedConfigFile(){
        return this.loadedConfigFile;
    }
    
    private List getMapBinSpinnerList(){
        return this.mapBinSpinnerList;
    }
    
    private List getRpmBinSpinnerList(){
        return this.rpmBinSpinnerList;
    }
    
    private List getIgnitionAdvanceSpinnerList(){
        return this.ignitionAdvanceSpinnerList;
    }
    
    private List getUserOutputConfigList(){
        return this.userOutputConfigList;
    }
    
    private void initUserOutputComponents(){
        
        List userOptionConfigs=getUserOutputConfigList();
        
        for (int index=0;index<6;index++){
            UserOutputConfigPanel optionConfig=new UserOutputConfigPanel();
            optionConfig.setName("Output " + String.valueOf(index+1));
            this.userOutputsGridPanel.add(optionConfig);
            userOptionConfigs.add(optionConfig);
        }
        
        UserOutputConfigPanel shiftLightConfig=(UserOutputConfigPanel)userOptionConfigs.get(4);
        shiftLightConfig.setName("Shift Light");
        shiftLightConfig.setSelectableType(false);
        shiftLightConfig.setType(IgnitionConfig.USER_OUTPUT_TYPE_RPM);
        
        UserOutputConfigPanel revLimitConfig=(UserOutputConfigPanel)userOptionConfigs.get(5);
        revLimitConfig.setName("Rev Limiter");
        revLimitConfig.setSelectableType(false);
        revLimitConfig.setType(IgnitionConfig.USER_OUTPUT_TYPE_RPM);
        
    }
    
    
    private void initIgnitionTableComponents(){

        JPanel gridPanel=this.ignitionGridPanel;
        List ignitionAdvanceSpinners=getRpmBinSpinnerList();
        
        //add the spinner components
        //set the upper left hand corner
        gridPanel.add(Box.createRigidArea(new Dimension(1,1)));
        //set the grid values
        for (int index=1; index < ((IGNITION_GRID_ROWS+1)*(IGNITION_GRID_COLS+1));index++){
            gridPanel.add(new JSpinner());
        }
        
        //update the spinner settings
        //set the RPM bin spinners
        List rpmBinSpinners=getRpmBinSpinnerList();
        for (int index=0;index < IGNITION_GRID_COLS;index++){
            JSpinner rpmBinSpinner=(JSpinner)gridPanel.getComponent(index+1);
            
            //set the model
            SpinnerModel model = new SpinnerNumberModel(MIN_RPM_VALUE,MIN_RPM_VALUE,MAX_RPM_VALUE,RPM_STEP_VALUE);
            rpmBinSpinner.setModel(model);

            //set the background color
            JSpinner.DefaultEditor e = (JSpinner.DefaultEditor)rpmBinSpinner.getEditor();
            e.getTextField().setBackground(RPM_BIN_BACKGROUND_COLOR);

            rpmBinSpinners.add(rpmBinSpinner);
        }
        
        //set the MAP bin spinners
        List mapBinSpinners=getMapBinSpinnerList();
        for (int index=0;index < IGNITION_GRID_ROWS;index++){
            JSpinner mapBinSpinner=(JSpinner)gridPanel.getComponent((index+1)*(IGNITION_GRID_COLS+1));

            //set the model
            SpinnerModel model = new SpinnerNumberModel(MIN_MAP_VALUE,MIN_MAP_VALUE,MAX_MAP_VALUE,MAP_STEP_VALUE);
            mapBinSpinner.setModel(model);
            
            //set the background color
            JSpinner.DefaultEditor e = (JSpinner.DefaultEditor)mapBinSpinner.getEditor();
            e.getTextField().setBackground(MAP_BIN_BACKGROUND_COLOR);
            
            mapBinSpinners.add(mapBinSpinner);
        }
        
        //set the ignition advance spinners
        
        ignitionAdvanceSpinners=getIgnitionAdvanceSpinnerList();
        
        for (int rowIndex=0; rowIndex<IGNITION_GRID_ROWS;rowIndex++){
            for (int colIndex=0;colIndex<IGNITION_GRID_COLS;colIndex++){
        
                JSpinner advanceSpinner=(JSpinner)gridPanel.getComponent(((rowIndex+1)*(IGNITION_GRID_COLS+1))+(colIndex+1));

                SpinnerModel model = new SpinnerNumberModel(MIN_ADVANCE_VALUE,MIN_ADVANCE_VALUE,MAX_ADVANCE_VALUE,ADVANCE_STEP_VALUE);
                advanceSpinner.setModel(model);

                JSpinner.DefaultEditor e = (JSpinner.DefaultEditor)advanceSpinner.getEditor();
                e.getTextField().setBackground(ADVANCE_BACKGROUND_COLOR);

                ignitionAdvanceSpinners.add(advanceSpinner);
                
            }
        }
        
        
    }
    
    private void reloadConfiguration(){
        
    try{
        IgnitionConfig config = readConfiguration();
        if (null!=config){
            updateConfiguration(config);
        }
    }
    catch (CommRuntimeException e){
        updateStatus(e.getMessage());
    }
    catch (Throwable t){
        ErrorDialog.showErrorDialog("There was a problem reading the configuration",t);
    }
    }
    
    private JFileChooser getFileChooser(){
        
        Preferences p = getPreferences();
      
        //load the previously used path
        String lastDirPath = p.get(LAST_DIRECTORY_PATH,null);
        if (null!=lastDirPath){
            File dir = new File(lastDirPath);
            if (dir.isDirectory()){
                this.fileChooser.setCurrentDirectory(dir);
            }
        }
        
        return this.fileChooser;
 
    }
    
    private void saveFileChooserSettings(){
        Preferences p = getPreferences();
        p.put(LAST_DIRECTORY_PATH, fileChooser.getCurrentDirectory().getAbsolutePath());
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        fileChooser = new javax.swing.JFileChooser();
        configTabbedPane = new javax.swing.JTabbedPane();
        ignitionTableConfigPanel = new javax.swing.JPanel();
        rpmBinsLabel = new javax.swing.JLabel();
        mapBinsLabel = new javax.swing.JLabel();
        ignitionGridPanel = new javax.swing.JPanel();
        ignitionGridSpacerPanel = new javax.swing.JPanel();
        userOutputsPanel = new javax.swing.JPanel();
        userOutputsGridPanel = new javax.swing.JPanel();
        titlePanel = new javax.swing.JPanel();
        leftSpacerPanel = new javax.swing.JPanel();
        typeLabel = new javax.swing.JLabel();
        modeLabel = new javax.swing.JLabel();
        triggerLabel = new javax.swing.JLabel();
        rightSpacer = new javax.swing.JPanel();
        generalOptionsPanel = new javax.swing.JPanel();
        cylindersLabel = new javax.swing.JLabel();
        cylindersComboBox = new javax.swing.JComboBox();
        statusLabel = new javax.swing.JLabel();
        buttonPanel = new javax.swing.JPanel();
        getConfigButton = new javax.swing.JButton();
        updateConfigButton = new javax.swing.JButton();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        openMenuItem = new javax.swing.JMenuItem();
        saveMenuItem = new javax.swing.JMenuItem();
        saveAsMenuItem = new javax.swing.JMenuItem();
        exitMenuItem = new javax.swing.JMenuItem();
        tools = new javax.swing.JMenu();
        runtimeDisplayMenuItem = new javax.swing.JMenuItem();
        optionsMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();

        fileChooser.setDialogTitle("Ignition Configuration");

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("MegaJolt Lite Jr");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        configTabbedPane.setFont(new java.awt.Font("Dialog", 0, 10));
        ignitionTableConfigPanel.setLayout(new java.awt.GridBagLayout());

        rpmBinsLabel.setText("RPM");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        ignitionTableConfigPanel.add(rpmBinsLabel, gridBagConstraints);

        mapBinsLabel.setText("MAP");
        ignitionTableConfigPanel.add(mapBinsLabel, new java.awt.GridBagConstraints());

        ignitionGridPanel.setLayout(new java.awt.GridLayout(11, 11, 2, 2));

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        ignitionTableConfigPanel.add(ignitionGridPanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        ignitionTableConfigPanel.add(ignitionGridSpacerPanel, gridBagConstraints);

        configTabbedPane.addTab("Ignition Table", ignitionTableConfigPanel);

        userOutputsPanel.setLayout(new java.awt.GridBagLayout());

        userOutputsGridPanel.setLayout(new java.awt.GridLayout(0, 1, 0, 3));

        titlePanel.setLayout(new java.awt.GridLayout(1, 5, 5, 0));

        titlePanel.add(leftSpacerPanel);

        typeLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        typeLabel.setText("Type");
        titlePanel.add(typeLabel);

        modeLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        modeLabel.setText("Mode");
        titlePanel.add(modeLabel);

        triggerLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        triggerLabel.setText("Trigger");
        titlePanel.add(triggerLabel);

        titlePanel.add(rightSpacer);

        userOutputsGridPanel.add(titlePanel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        userOutputsPanel.add(userOutputsGridPanel, gridBagConstraints);

        configTabbedPane.addTab("Programmable Outputs", userOutputsPanel);

        generalOptionsPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        cylindersLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        cylindersLabel.setText("Number of Cylinders");
        generalOptionsPanel.add(cylindersLabel);

        cylindersComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "4", "6", "8" }));
        cylindersComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cylindersComboBoxActionPerformed(evt);
            }
        });

        generalOptionsPanel.add(cylindersComboBox);

        configTabbedPane.addTab("General", generalOptionsPanel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 2.0;
        getContentPane().add(configTabbedPane, gridBagConstraints);

        statusLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        statusLabel.setForeground(new java.awt.Color(255, 0, 51));
        statusLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(statusLabel, gridBagConstraints);

        buttonPanel.setLayout(new java.awt.GridBagLayout());

        getConfigButton.setFont(new java.awt.Font("Dialog", 0, 10));
        getConfigButton.setText("Get Config");
        getConfigButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                getConfigButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        buttonPanel.add(getConfigButton, gridBagConstraints);

        updateConfigButton.setFont(new java.awt.Font("Dialog", 0, 10));
        updateConfigButton.setText("Update Config");
        updateConfigButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateConfigButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        buttonPanel.add(updateConfigButton, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(buttonPanel, gridBagConstraints);

        fileMenu.setText("File");
        fileMenu.setFont(new java.awt.Font("Dialog", 0, 12));
        openMenuItem.setFont(new java.awt.Font("Dialog", 0, 12));
        openMenuItem.setText("Open");
        openMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(openMenuItem);

        saveMenuItem.setFont(new java.awt.Font("Dialog", 0, 12));
        saveMenuItem.setText("Save");
        saveMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(saveMenuItem);

        saveAsMenuItem.setFont(new java.awt.Font("Dialog", 0, 12));
        saveAsMenuItem.setText("Save As ...");
        saveAsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveAsMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(saveAsMenuItem);

        exitMenuItem.setFont(new java.awt.Font("Dialog", 0, 12));
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        tools.setText("Tools");
        tools.setFont(new java.awt.Font("Dialog", 0, 12));
        runtimeDisplayMenuItem.setFont(new java.awt.Font("Dialog", 0, 12));
        runtimeDisplayMenuItem.setText("Runtime Display");
        runtimeDisplayMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                runtimeDisplayMenuItemActionPerformed(evt);
            }
        });

        tools.add(runtimeDisplayMenuItem);

        optionsMenuItem.setFont(new java.awt.Font("Dialog", 0, 12));
        optionsMenuItem.setText("Options");
        optionsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                optionsMenuItemActionPerformed(evt);
            }
        });

        tools.add(optionsMenuItem);

        menuBar.add(tools);

        helpMenu.setText("Help");
        helpMenu.setFont(new java.awt.Font("Dialog", 0, 12));
        aboutMenuItem.setFont(new java.awt.Font("Dialog", 0, 12));
        aboutMenuItem.setText("About");
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setJMenuBar(menuBar);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-740)/2, (screenSize.height-440)/2, 740, 440);
    }//GEN-END:initComponents

    private void saveAsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveAsMenuItemActionPerformed
        // Add your handling code here:
        JFileChooser fc=getFileChooser();
 
        fc.showSaveDialog(this);
        File f = fc.getSelectedFile();

        if (null!=f){
            saveFileChooserSettings();
            IgnitionConfigFileIO icIO = new IgnitionConfigFileIO();
            IgnitionConfig ic = createConfiguration();

            try{
                icIO.writeIgnitionConfig(f, ic);

                setLoadedConfigFile(f);
            }
            catch (FileNotFoundException e){
                ErrorDialog.showErrorDialog("Could not write Ignition Configuration to file " + f.toString(),e);
            }
            catch (IOException e){
                ErrorDialog.showErrorDialog("Could not write Ignition Configuration", e);
            }
        }
        
    }//GEN-LAST:event_saveAsMenuItemActionPerformed

    private void saveMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveMenuItemActionPerformed
        if (null==getLoadedConfigFile()) saveAsMenuItemActionPerformed(evt);
    }//GEN-LAST:event_saveMenuItemActionPerformed

    private void openMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openMenuItemActionPerformed
        
        JFileChooser fc = getFileChooser();
        
        fc.showOpenDialog(this);

        File f = fc.getSelectedFile();
        if (null!=f){
            //save the current directory
            saveFileChooserSettings();
            
            //create the ignition config
            IgnitionConfigFileIO icIO = new IgnitionConfigFileIO();
        
            try{
                IgnitionConfig config = icIO.readIgnitionConfig(f);
                if (null!=config){
                    updateConfiguration(config);
                    setLoadedConfigFile(f);
                }
            }
            catch (FileNotFoundException e){
                ErrorDialog.showErrorDialog("Could not find Ignition Configuration file " + f.toString(),e);
            }
            catch (IOException e){
                ErrorDialog.showErrorDialog("Could not load Ignition Configuration",e);
            }
        }
            
        
    }//GEN-LAST:event_openMenuItemActionPerformed

    private void optionsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_optionsMenuItemActionPerformed
        // Add your handling code here:
        OptionsDialog dlg = new OptionsDialog(this,true);
        dlg.show();
        
    }//GEN-LAST:event_optionsMenuItemActionPerformed

    private void cylindersComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cylindersComboBoxActionPerformed
        // Add your handling code here:
    }//GEN-LAST:event_cylindersComboBoxActionPerformed

    private void updateConfigButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateConfigButtonActionPerformed
        // Add your handling code here:
        IgnitionConfig config = createConfiguration();
        sendConfiguration(config);
        writeFlash();
    }//GEN-LAST:event_updateConfigButtonActionPerformed

    private void getConfigButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_getConfigButtonActionPerformed
        // Add your handling code here:
    
        reloadConfiguration();
        
    }//GEN-LAST:event_getConfigButtonActionPerformed

    private void runtimeDisplayMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_runtimeDisplayMenuItemActionPerformed
        // Add your handling code here:
        RuntimeDisplay runtimeDisplay = new RuntimeDisplay(this, true);
        runtimeDisplay.setIgnitionController(getIgnitionController());
        
        runtimeDisplay.show();
        
    }//GEN-LAST:event_runtimeDisplayMenuItemActionPerformed
    
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        System.exit(0);
    }//GEN-LAST:event_exitMenuItemActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new MegaJoltLiteJr().show();
    }
    
    
    public IgnitionConfig createConfiguration(){

        IgnitionConfig config = new IgnitionConfig();

        List rpmBinSpinners = getRpmBinSpinnerList();
        List mapBinSpinners = getMapBinSpinnerList();
        List ignitionAdvanceSpinners = getIgnitionAdvanceSpinnerList();
        
        config.setCylinderCount(getCylinders());

        for (int index = 0;index<IGNITION_GRID_COLS;index++){
            JSpinner rpmBinSpinner = (JSpinner)rpmBinSpinners.get(index);
            config.setRpmBinValue(index, ((Integer)rpmBinSpinner.getValue()).intValue()/100);
        }
        
        for (int index = 0; index<IGNITION_GRID_ROWS;index++){
            JSpinner mapBinSpinner = (JSpinner)mapBinSpinners.get(index);
            config.setMapBinValue(index, ((Integer)mapBinSpinner.getValue()).intValue());
        }

        for (int rowIndex=0; rowIndex<IGNITION_GRID_ROWS;rowIndex++){
            for (int colIndex=0;colIndex<IGNITION_GRID_COLS;colIndex++){
                
                JSpinner ignitionAdvanceSpinner=(JSpinner)ignitionAdvanceSpinners.get((rowIndex*IGNITION_GRID_ROWS)+colIndex);
                config.setIgnitionMapValue(colIndex, rowIndex, ((Integer)ignitionAdvanceSpinner.getValue()).intValue());
            }
        }
        
        List userOutputConfigs = getUserOutputConfigList();
        
        UserOutputConfigPanel shiftLightConfig = (UserOutputConfigPanel)userOutputConfigs.get(4);
        UserOutputConfigPanel revLimitConfig = (UserOutputConfigPanel)userOutputConfigs.get(5);
        
        config.setShiftLightRpm(shiftLightConfig.getTrigger()/100);
        config.setRevLimitRpm(revLimitConfig.getTrigger()/100);
        
        for (int userOutputIndex=0;userOutputIndex<4;userOutputIndex++){
            UserOutputConfigPanel userOutputConfig = (UserOutputConfigPanel)userOutputConfigs.get(userOutputIndex);
            config.setUserOutputType(userOutputIndex, userOutputConfig.getType());
            config.setUserOutputMode(userOutputIndex, userOutputConfig.getMode());
            
            if (userOutputConfig.getType()==IgnitionConfig.USER_OUTPUT_TYPE_RPM){
                config.setUserOutputValue(userOutputIndex, userOutputConfig.getTrigger()/100);
            }
            else{
                config.setUserOutputValue(userOutputIndex, userOutputConfig.getTrigger());                
            }
        }
        
        config.setCylinderCount(getCylinders());

        return config;
    }
    
    public void sendConfiguration(IgnitionConfig config){
 
        try{
            IgnitionController controller = getIgnitionController();

            controller.updateIgnitionConfig(config);
        }
        catch(Throwable t){
            ErrorDialog.showErrorDialog("There was an error updating the configuration: " + t.getMessage(),t);
        }
        
    }
    
    public void writeFlash(){
    
        try{
            IgnitionController controller = getIgnitionController();
            controller.writeFlash();
        }
        catch (Throwable t){
            ErrorDialog.showErrorDialog("There was an error writing flash: " + t.getMessage(),t);
        }
    }
    
    public IgnitionConfig readConfiguration(){

        IgnitionController controller = getIgnitionController();
        try{
            
            IgnitionConfig config = controller.getIgnitionConfig();
            updateStatus("");
            return config;
        }
        catch (CommRuntimeException e){
            updateStatus(e.getMessage());
            return null;
        }
        catch (NoControllerConnectionException e){
            updateStatus(e.getMessage());
            return null;
        }
        catch( Throwable t){
            ErrorDialog.showErrorDialog("There was an error reading the configuration: " + t.getMessage(),t);
            updateStatus("Error reading configuration");
            return null;
        }
        finally{
        }
    }
    
    public void updateConfiguration(IgnitionConfig config){
        
        List rpmBinSpinners = getRpmBinSpinnerList();
        List mapBinSpinners = getMapBinSpinnerList();
        List ignitionAdvanceSpinners = getIgnitionAdvanceSpinnerList();
        
        for (int index = 0;index<IGNITION_GRID_COLS;index++){
            JSpinner rpmBinSpinner = (JSpinner)rpmBinSpinners.get(index);
            rpmBinSpinner.setValue(new Integer(config.getRpmBinValue(index)*100));
        }
        
        for (int index = 0; index<IGNITION_GRID_ROWS;index++){
            JSpinner mapBinSpinner = (JSpinner)mapBinSpinners.get(index);
            mapBinSpinner.setValue(new Integer(config.getMapBinValue(index)));
        }
        
        for (int rowIndex=0; rowIndex<IGNITION_GRID_ROWS;rowIndex++){
            for (int colIndex=0;colIndex<IGNITION_GRID_COLS;colIndex++){
                
                JSpinner ignitionAdvanceSpinner=(JSpinner)ignitionAdvanceSpinners.get((rowIndex*IGNITION_GRID_COLS)+colIndex);
                ignitionAdvanceSpinner.setValue(new Integer(config.getIgnitionMapValue(colIndex,rowIndex)));
            }
        }
        
        List userOutputConfigs = getUserOutputConfigList();
        UserOutputConfigPanel shiftLightConfig = (UserOutputConfigPanel)userOutputConfigs.get(4);
        UserOutputConfigPanel revLimitConfig = (UserOutputConfigPanel)userOutputConfigs.get(5);
        
        shiftLightConfig.setTrigger(config.getShiftLightRpm()*100);
        revLimitConfig.setTrigger(config.getRevLimitRpm()*100);
        
        for (int index=0;index<4;index++){
            
            UserOutputConfigPanel configPanel = (UserOutputConfigPanel)userOutputConfigs.get(index);
            configPanel.setMode(config.getUserOutputMode(index));
            int type=config.getUserOutputType(index);
            configPanel.setType(type);
            
            if (type==IgnitionConfig.USER_OUTPUT_TYPE_RPM){
                configPanel.setTrigger(config.getUserOutputValue(index)*100);
            }
            else{
                configPanel.setTrigger(config.getUserOutputValue(index));
            }
        }
        
        setCylinders(config.getCylinderCount());
        
    }
    
    private void updateStatus(String newStatus){
        
        this.statusLabel.setText(newStatus);
    }
    
    private void updateTitleBar(){
        
        File configFile = getLoadedConfigFile();
        
        String title = TITLE_BAR_TEXT + (null==configFile?" - (No file loaded)":" - " + configFile.getName());
        this.setTitle(title);
        
    }
    
    /** Getter for property cylinders.
     * @return Value of property cylinders.
     *
     */
    public int getCylinders() {
        return Integer.valueOf(this.cylindersComboBox.getSelectedItem().toString()).intValue();
    }
    
    /** Setter for property cylinders.
     * @param cylinders New value of property cylinders.
     *
     */
    public void setCylinders(int cylinders) {
        this.cylindersComboBox.setSelectedItem(String.valueOf(cylinders));
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JTabbedPane configTabbedPane;
    private javax.swing.JComboBox cylindersComboBox;
    private javax.swing.JLabel cylindersLabel;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JFileChooser fileChooser;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JPanel generalOptionsPanel;
    private javax.swing.JButton getConfigButton;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JPanel ignitionGridPanel;
    private javax.swing.JPanel ignitionGridSpacerPanel;
    private javax.swing.JPanel ignitionTableConfigPanel;
    private javax.swing.JPanel leftSpacerPanel;
    private javax.swing.JLabel mapBinsLabel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JLabel modeLabel;
    private javax.swing.JMenuItem openMenuItem;
    private javax.swing.JMenuItem optionsMenuItem;
    private javax.swing.JPanel rightSpacer;
    private javax.swing.JLabel rpmBinsLabel;
    private javax.swing.JMenuItem runtimeDisplayMenuItem;
    private javax.swing.JMenuItem saveAsMenuItem;
    private javax.swing.JMenuItem saveMenuItem;
    private javax.swing.JLabel statusLabel;
    private javax.swing.JPanel titlePanel;
    private javax.swing.JMenu tools;
    private javax.swing.JLabel triggerLabel;
    private javax.swing.JLabel typeLabel;
    private javax.swing.JButton updateConfigButton;
    private javax.swing.JPanel userOutputsGridPanel;
    private javax.swing.JPanel userOutputsPanel;
    // End of variables declaration//GEN-END:variables

    /** Holds value of property cylinders. */
    private int cylinders=DEFAULT_CYLINDERS;
    
}
