/*
 * RuntimeDisplay.java
 *
 * Created on May 17, 2003, 1:01 AM
 */

package org.megaJoltLiteJr.presentation;

/**
 *
 * @author  brent
 */
import org.megaJoltLiteJr.*;
import javax.swing.table.DefaultTableModel;
import org.megaJoltLiteJr.util.ControllerUtil;

public class RuntimeDisplay extends javax.swing.JDialog implements Runnable{
    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    public static final long POLL_INTERVAL=500;

    private int returnStatus = RET_CANCEL;
    /** Holds value of property shouldRun. */
    private boolean shouldRun;

    // the number of cylinders that's currently configured
    private int numberOfCylinders;
    private IgnitionController ignitionController;
    
    /** Creates new form RuntimeDisplay */
    public RuntimeDisplay(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
    }
    
    /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
    public int getReturnStatus() {
        return returnStatus;
    }

    /** Sets the ignition controller object
     * @param the ignition controller object
     */
    public void setIgnitionController(IgnitionController newIgnitionController){
        this.ignitionController=newIgnitionController;
    }
    
    /** Retrieves the current ignition controller object
     * @param the ignition controller object
     */
    public IgnitionController getIgnitionController(){
        return this.ignitionController;
    }
    
    /** Sets the currently configured number of Cylinders
     * @param the new number of cylinders value
     */
    protected void setNumberOfCylinders(int newNumberOfCylinders){
        this.numberOfCylinders=newNumberOfCylinders;
    }
    
    /** Retrieves the currently configured number of cylinders
     * @return the number of cylinders
     */
    protected int getNumberOfCylinders(){
        return this.numberOfCylinders;
    }
    
    public void show(){
     
        if (null==getIgnitionController()) {
            throw new IllegalStateException("Could not start runtime display: Ignition Controller not set");
        }
        startDisplay();
        super.show();
    }
    public void startDisplay(){
     
        Thread t = new Thread(this);
        setShouldRun(true);
        
        t.start();
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        RuntimeDisplayPanel = new javax.swing.JPanel();
        ignitionAdvanceLabel = new javax.swing.JLabel();
        ignitionAdvanceTextField = new javax.swing.JTextField();
        ignitionAdvanceBar = new javax.swing.JProgressBar();
        rpmValueLabel = new javax.swing.JLabel();
        rpmValueTextField = new javax.swing.JTextField();
        rpmValueBar = new javax.swing.JProgressBar();
        mapValueLabel = new javax.swing.JLabel();
        mapValueTextField = new javax.swing.JTextField();
        mapValueBar = new javax.swing.JProgressBar();
        ignitionMapPanel = new javax.swing.JPanel();
        ignitionMapTable = new javax.swing.JTable();
        statusLabel = new javax.swing.JLabel();

        setTitle("Runtime Display");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        okButton.setFont(new java.awt.Font("Dialog", 0, 10));
        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(okButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

        RuntimeDisplayPanel.setLayout(new java.awt.GridBagLayout());

        ignitionAdvanceLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        ignitionAdvanceLabel.setText("Advance");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        RuntimeDisplayPanel.add(ignitionAdvanceLabel, gridBagConstraints);

        ignitionAdvanceTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        RuntimeDisplayPanel.add(ignitionAdvanceTextField, gridBagConstraints);

        ignitionAdvanceBar.setMaximum(60);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        RuntimeDisplayPanel.add(ignitionAdvanceBar, gridBagConstraints);

        rpmValueLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        rpmValueLabel.setText("RPM");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        RuntimeDisplayPanel.add(rpmValueLabel, gridBagConstraints);

        rpmValueTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        RuntimeDisplayPanel.add(rpmValueTextField, gridBagConstraints);

        rpmValueBar.setMaximum(10000);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        RuntimeDisplayPanel.add(rpmValueBar, gridBagConstraints);

        mapValueLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        mapValueLabel.setText("MAP KPa");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        RuntimeDisplayPanel.add(mapValueLabel, gridBagConstraints);

        mapValueTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        RuntimeDisplayPanel.add(mapValueTextField, gridBagConstraints);

        mapValueBar.setMaximum(255);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        RuntimeDisplayPanel.add(mapValueBar, gridBagConstraints);

        ignitionMapPanel.setLayout(new java.awt.BorderLayout());

        ignitionMapPanel.setBorder(new javax.swing.border.TitledBorder(null, "Ignition Map", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 0, 10)));
        ignitionMapTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4", "Title 5", "Title 6", "Title 7", "Title 8", "Title 9", "Title 10", "Title 11"
            }
        ));
        ignitionMapTable.setCellSelectionEnabled(true);
        ignitionMapTable.setSelectionBackground(new java.awt.Color(255, 255, 0));
        ignitionMapTable.setEnabled(false);
        ignitionMapPanel.add(ignitionMapTable, java.awt.BorderLayout.CENTER);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        RuntimeDisplayPanel.add(ignitionMapPanel, gridBagConstraints);

        statusLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        statusLabel.setForeground(new java.awt.Color(255, 0, 51));
        statusLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        RuntimeDisplayPanel.add(statusLabel, gridBagConstraints);

        getContentPane().add(RuntimeDisplayPanel, java.awt.BorderLayout.CENTER);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-513)/2, (screenSize.height-355)/2, 513, 355);
    }//GEN-END:initComponents
    
    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        doClose(RET_OK);
    }//GEN-LAST:event_okButtonActionPerformed
        
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        doClose(RET_CANCEL);
    }//GEN-LAST:event_closeDialog
    
    private void doClose(int retStatus) {
        returnStatus = retStatus;
        setShouldRun(false);
        setVisible(false);
        dispose();
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new RuntimeDisplay(new javax.swing.JFrame(), true).show();
    }
    
    public void run() {


        IgnitionState currentState=null;
        IgnitionController controller = getIgnitionController();

        
        try{

            IgnitionConfig config = controller.getIgnitionConfig();
            setIgnitionMapView(config);
            setNumberOfCylinders(config.getCylinderCount());
        }
        catch (Throwable t){
            setStatusMessage("Unable to communicate with controller");
            System.err.println(t.getMessage());
            return;
        }
        
        while(getShouldRun()){
        
            try{
                currentState= controller.getIgnitionState();
                updateRuntimeDisplay(currentState);
                clearStatusMessage();
            }
            catch(Throwable t){
                setStatusMessage("Unable to communicate with controller");
                System.err.println(t.getMessage());
            }
            try{
                Thread.sleep(POLL_INTERVAL);
            }
            catch(InterruptedException e){}
            
        }
        
   }
    
    
    private void setIgnitionMapView(IgnitionConfig config){
    
        DefaultTableModel m = (DefaultTableModel)this.ignitionMapTable.getModel();
        
        
        for (int mapIndex=0;mapIndex<IgnitionConfig.MAP_BIN_SIZE;mapIndex++){
            m.setValueAt(String.valueOf(config.getMapBinValue(mapIndex)), mapIndex+1,0);
        }
        for (int rpmIndex=0;rpmIndex<IgnitionConfig.RPM_BIN_SIZE;rpmIndex++){
            m.setValueAt(String.valueOf(ControllerUtil.convertBinToRpm(config.getRpmBinValue(rpmIndex))), 0 , rpmIndex+1);
        }

        for (int mapIndex=0;mapIndex<IgnitionConfig.MAP_BIN_SIZE;mapIndex++){
            for (int rpmIndex=0;rpmIndex<IgnitionConfig.RPM_BIN_SIZE;rpmIndex++){
                m.setValueAt(String.valueOf(config.getIgnitionMapValue(rpmIndex,mapIndex)), mapIndex+1,rpmIndex+1);
            }
        }
        
    }
    
    private void updateRuntimeDisplay(IgnitionState state){
     
        this.ignitionAdvanceTextField.setText(String.valueOf(state.getIgnitionAdvance()));
        this.ignitionAdvanceBar.setValue(state.getIgnitionAdvance());

        int rpm = ControllerUtil.convertTicksToRpm(state.getRpmValue(),getNumberOfCylinders());
        
        this.rpmValueTextField.setText(String.valueOf(rpm));
        
        this.rpmValueBar.setValue(rpm);

        this.mapValueTextField.setText(String.valueOf(state.getMapValue()));
        this.mapValueBar.setValue(state.getMapValue());
        
        DefaultTableModel m = (DefaultTableModel)this.ignitionMapTable.getModel();
        
        
//        m.setValueAt(String.valueOf(state.getIgnitionAdvance()), state.getMapBin(),state.getRpmBin());
        this.ignitionMapTable.changeSelection(state.getMapBin()+1,state.getRpmBin()+1, false, false);
        
    }
    /** Getter for property shouldRun.
     * @return Value of property shouldRun.
     *
     */
    public boolean getShouldRun() {
        return this.shouldRun;
    }
    
    /** Setter for property shouldRun.
     * @param shouldRun New value of property shouldRun.
     *
     */
    public void setShouldRun(boolean shouldRun) {
        this.shouldRun = shouldRun;
    }
    
    /** Getter for property lastMapBin.
     * @return Value of property lastMapBin.
     *
     */
    public int getLastMapBin() {
        return this.lastMapBin;
    }
    
    /** Setter for property lastMapBin.
     * @param lastMapBin New value of property lastMapBin.
     *
     */
    public void setLastMapBin(int lastMapBin) {
        this.lastMapBin = lastMapBin;
    }
    
    /** Getter for property lastRpmBin.
     * @return Value of property lastRpmBin.
     *
     */
    public int getLastRpmBin() {
        return this.lastRpmBin;
    }

    public void clearStatusMessage(){
        this.statusLabel.setText("");
    }
    
    public void setStatusMessage(String statusMsg){
        this.statusLabel.setText(statusMsg);
    }
    
    /** Setter for property lastRpmBin.
     * @param lastRpmBin New value of property lastRpmBin.
     *
     */
    public void setLastRpmBin(int lastRpmBin) {
        this.lastRpmBin = lastRpmBin;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel RuntimeDisplayPanel;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JProgressBar ignitionAdvanceBar;
    private javax.swing.JLabel ignitionAdvanceLabel;
    private javax.swing.JTextField ignitionAdvanceTextField;
    private javax.swing.JPanel ignitionMapPanel;
    private javax.swing.JTable ignitionMapTable;
    private javax.swing.JProgressBar mapValueBar;
    private javax.swing.JLabel mapValueLabel;
    private javax.swing.JTextField mapValueTextField;
    private javax.swing.JButton okButton;
    private javax.swing.JProgressBar rpmValueBar;
    private javax.swing.JLabel rpmValueLabel;
    private javax.swing.JTextField rpmValueTextField;
    private javax.swing.JLabel statusLabel;
    // End of variables declaration//GEN-END:variables

    /** Holds value of property lastMapBin. */
    private int lastMapBin;    
    
    /** Holds value of property lastRpmBin. */
    private int lastRpmBin;    
    
}
