/*
 * UserOptionConfigPanel.java
 *
 * Created on June 9, 2003, 6:51 PM
 */

package org.megaJoltLiteJr.presentation;

import org.megaJoltLiteJr.*;
import javax.swing.*;
import java.awt.event.ItemListener;

/**
 *
 * @author  brent
 */
public class UserOutputConfigPanel extends javax.swing.JPanel implements ItemListener{
    
    
    private static final int MIN_RPM_VALUE=IgnitionConfig.MIN_RPM_BIN_VALUE*100;
    private static final int MAX_RPM_VALUE=IgnitionConfig.MAX_RPM_BIN_VALUE*100;
    private static final int RPM_STEP_VALUE=100;
    private static final int MIN_MAP_VALUE=IgnitionConfig.MIN_MAP_BIN_VALUE;
    private static final int MAX_MAP_VALUE=IgnitionConfig.MAX_MAP_BIN_VALUE;
    private static final int MAP_STEP_VALUE=1;
    
    /** Creates new form UserOptionConfigPanel */
    public UserOutputConfigPanel() {
        initComponents();
        initComponentSettings();
        initListeners();
    }
    
    private void initListeners(){
     
        this.typeComboBox.addItemListener(this);
    }
    
    private void initComponentSettings(){
     
        SpinnerModel model = new SpinnerNumberModel(MIN_RPM_VALUE,MIN_RPM_VALUE,MAX_RPM_VALUE,RPM_STEP_VALUE);
        this.triggerSpinner.setModel(model);
        
        //set some defaults
        setTrigger(5000);
        setMode(IgnitionConfig.USER_OUTPUT_MODE_HIGH);
        setType(IgnitionConfig.USER_OUTPUT_TYPE_RPM);
        setSelectableType(true);
        setName("User Output");
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        nameLabel = new javax.swing.JLabel();
        typeComboBox = new javax.swing.JComboBox();
        modeComboBox = new javax.swing.JComboBox();
        triggerSpinner = new javax.swing.JSpinner();
        unitsLabel = new javax.swing.JLabel();

        setLayout(new java.awt.GridLayout(0, 5, 5, 1));

        nameLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        nameLabel.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        nameLabel.setText("output");
        add(nameLabel);

        typeComboBox.setFont(new java.awt.Font("Dialog", 0, 12));
        typeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "RPM", "MAP" }));
        typeComboBox.setToolTipText("Sets the type of switched output- RPM or Manifold Pressure ");
        typeComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                typeComboBoxActionPerformed(evt);
            }
        });

        add(typeComboBox);

        modeComboBox.setFont(new java.awt.Font("Dialog", 0, 12));
        modeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "High", "Low" }));
        modeComboBox.setToolTipText("Sets whether the output goes high or low when trigger threshold is exceeded");
        add(modeComboBox);

        triggerSpinner.setToolTipText("sets the threshold trigger value for this output");
        add(triggerSpinner);

        unitsLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        unitsLabel.setText("rpm");
        add(unitsLabel);

    }//GEN-END:initComponents

    private void typeComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_typeComboBoxActionPerformed
        // Add your handling code here:
    }//GEN-LAST:event_typeComboBoxActionPerformed
    
    
    public int getMode(){
        return this.modeComboBox.getSelectedItem().equals("High")?IgnitionConfig.USER_OUTPUT_MODE_HIGH:IgnitionConfig.USER_OUTPUT_MODE_LOW;
    }
    
    public void setMode(int newMode){
        this.modeComboBox.setSelectedItem(newMode==IgnitionConfig.USER_OUTPUT_MODE_HIGH?"High":"Low");
    }
    
    public void setType(int newType){
        this.typeComboBox.setSelectedItem(newType==IgnitionConfig.USER_OUTPUT_TYPE_MAP?"MAP":"RPM");
        setSpinnerType(newType);
    }
    
    private void setSpinnerType(int newType){

        SpinnerNumberModel m = (SpinnerNumberModel)this.triggerSpinner.getModel();
        if (IgnitionConfig.USER_OUTPUT_TYPE_RPM==newType){
            m.setMinimum(new Integer(MIN_RPM_VALUE));
            m.setMaximum(new Integer(MAX_RPM_VALUE));
            m.setStepSize(new Integer(RPM_STEP_VALUE));
            if (((Integer)m.getValue()).intValue()>MAX_RPM_VALUE) m.setValue(new Integer(MAX_RPM_VALUE));
            if (((Integer)m.getValue()).intValue()<MIN_RPM_VALUE) m.setValue(new Integer(MIN_RPM_VALUE));
            this.unitsLabel.setText("RPM");
        }
        else{
            m.setMinimum(new Integer(MIN_MAP_VALUE));
            m.setMaximum(new Integer(MAX_MAP_VALUE));
            m.setStepSize(new Integer(MAP_STEP_VALUE));
            if (((Integer)m.getValue()).intValue()>MAX_MAP_VALUE) m.setValue(new Integer(MAX_MAP_VALUE));
            if (((Integer)m.getValue()).intValue()<MIN_MAP_VALUE) m.setValue(new Integer(MIN_MAP_VALUE));
            this.unitsLabel.setText("KPa");
        }
    }
    
    public int getType(){
        return this.typeComboBox.getSelectedItem().equals("MAP")?IgnitionConfig.USER_OUTPUT_TYPE_MAP:IgnitionConfig.USER_OUTPUT_TYPE_RPM;
    }
    
    public int getTrigger(){
        return ((Integer)this.triggerSpinner.getValue()).intValue();
    }
    
    public void setTrigger(int newTrigger){
        this.triggerSpinner.setValue(new Integer(newTrigger));
    }
    
    public void setName(String newName){
        this.nameLabel.setText(newName);
    }
    
    public String getName(){
        return this.nameLabel.getText();
    }
    
    /** Getter for property selectableType.
     * @return Value of property selectableType.
     *
     */
    public boolean isSelectableType() {
        return this.selectableType;
    }    
   
    /** Setter for property selectableType.
     * @param selectableType New value of property selectableType.
     *
     */
    public void setSelectableType(boolean selectableType) {
        this.selectableType = selectableType;
        this.typeComboBox.setVisible(selectableType);
        this.modeComboBox.setVisible(selectableType);
    }    
    
    public void itemStateChanged(java.awt.event.ItemEvent e) {
        
        setSpinnerType(getType());
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox modeComboBox;
    private javax.swing.JLabel nameLabel;
    private javax.swing.JSpinner triggerSpinner;
    private javax.swing.JComboBox typeComboBox;
    private javax.swing.JLabel unitsLabel;
    // End of variables declaration//GEN-END:variables

    /** Holds value of property selectableType. */
    private boolean selectableType;    
    
}
